@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -41,7 +42,7 @@ namespace Orthanc
   {
     if (!source.HasTag(tag))
     {
-      throw OrthancException(ErrorCode_BadRequest);
+      throw OrthancException(ErrorCode_BadRequest, "Missing tag " + tag.Format());
     }
     else
     {
@@ -70,13 +71,27 @@ namespace Orthanc
     DcmDataset *responseIdentifiers /* pending response identifiers */
     )
   {
-    FindPayload& payload = *reinterpret_cast<FindPayload*>(callbackData);
-
+    if (response != NULL)
+    {
+      OFString str;
+      CLOG(TRACE, DICOM) << "Received Find Response " << responseCount << ":" << std::endl
+                         << DIMSE_dumpMessage(str, *response, DIMSE_INCOMING);
+    }
+      
     if (responseIdentifiers != NULL)
     {
+      std::stringstream s;  // DcmObject::PrintHelper cannot be used with VS2008
+      responseIdentifiers->print(s);
+      CLOG(TRACE, DICOM) << "Response Identifiers "  << responseCount << ":" << std::endl << s.str();
+    }
+    
+    if (responseIdentifiers != NULL)
+    {
+      FindPayload& payload = *reinterpret_cast<FindPayload*>(callbackData);
+
       if (payload.isWorklist)
       {
-        ParsedDicomFile answer(*responseIdentifiers);
+        const ParsedDicomFile answer(*responseIdentifiers);
         payload.answers->Add(answer);
       }
       else
@@ -153,7 +168,8 @@ namespace Orthanc
       const DicomTag& tag = query.GetElement(i).GetTag();
       if (allowedTags.find(tag) == allowedTags.end())
       {
-        LOG(WARNING) << "Tag not allowed for this C-Find level, will be ignored: " << tag;
+        CLOG(WARNING, DICOM) << "Tag not allowed for this C-Find level, will be ignored: ("
+                             << tag.Format() << ")";
       }
       else
       {
@@ -173,7 +189,7 @@ namespace Orthanc
     // same fix is required for Agfa Impax. This was generalized for
     // generic manufacturer since it seems to affect PhilipsADW,
     // GEWAServer as well:
-    // https://bitbucket.org/sjodogne/orthanc/issues/31/
+    // https://bugs.orthanc-server.com/show_bug.cgi?id=31
 
     switch (manufacturer)
     {
@@ -222,6 +238,7 @@ namespace Orthanc
     assert(association_.get() != NULL);
     association_->ProposeGenericPresentationContext(UID_VerificationSOPClass);
     association_->ProposeGenericPresentationContext(UID_FINDPatientRootQueryRetrieveInformationModel);
+    association_->ProposeGenericPresentationContext(UID_MOVEPatientRootQueryRetrieveInformationModel);
     association_->ProposeGenericPresentationContext(UID_FINDStudyRootQueryRetrieveInformationModel);
     association_->ProposeGenericPresentationContext(UID_MOVEStudyRootQueryRetrieveInformationModel);
     association_->ProposeGenericPresentationContext(UID_FINDModalityWorklistInformationModel);
@@ -234,6 +251,7 @@ namespace Orthanc
                                                 bool isWorklist,
                                                 const char* level)
   {
+    assert(dataset != NULL);
     assert(isWorklist ^ (level != NULL));
     assert(association_.get() != NULL);
 
@@ -267,6 +285,16 @@ namespace Orthanc
     int responseCount;
 #endif
 
+    {
+      std::stringstream s;  // DcmObject::PrintHelper cannot be used with VS2008
+      dataset->print(s);
+
+      OFString str;
+      CLOG(TRACE, DICOM) << "Sending Find Request:" << std::endl
+                         << DIMSE_dumpMessage(str, request, DIMSE_OUTGOING, NULL, presID) << std::endl
+                         << s.str();
+    }
+
     OFCondition cond = DIMSE_findUser(
       &association_->GetDcmtkAssociation(), presID, &request, dataset,
 #if DCMTK_VERSION_NUMBER >= 364
@@ -284,6 +312,12 @@ namespace Orthanc
 
     DicomAssociation::CheckCondition(cond, parameters_, "C-FIND");
 
+    {
+      OFString str;
+      CLOG(TRACE, DICOM) << "Received Final Find Response:" << std::endl
+                         << DIMSE_dumpMessage(str, response, DIMSE_INCOMING);
+    }
+
     
     /**
      * New in Orthanc 1.6.0: Deal with failures during C-FIND.
@@ -328,27 +362,7 @@ namespace Orthanc
     DcmDataset* dataset = query->GetDcmtkObject().getDataset();
 
     const char* sopClass = UID_MOVEStudyRootQueryRetrieveInformationModel;
-    switch (level)
-    {
-      case ResourceType_Patient:
-        DU_putStringDOElement(dataset, DCM_QueryRetrieveLevel, "PATIENT");
-        break;
-
-      case ResourceType_Study:
-        DU_putStringDOElement(dataset, DCM_QueryRetrieveLevel, "STUDY");
-        break;
-
-      case ResourceType_Series:
-        DU_putStringDOElement(dataset, DCM_QueryRetrieveLevel, "SERIES");
-        break;
-
-      case ResourceType_Instance:
-        DU_putStringDOElement(dataset, DCM_QueryRetrieveLevel, "IMAGE");
-        break;
-
-      default:
-        throw OrthancException(ErrorCode_ParameterOutOfRange);
-    }
+    DU_putStringDOElement(dataset, DCM_QueryRetrieveLevel, ResourceTypeToDicomQueryRetrieveLevel(level));
 
     // Figure out which of the accepted presentation contexts should be used
     int presID = ASC_findAcceptedPresentationContextID(&association_->GetDcmtkAssociation(), sopClass);
@@ -366,14 +380,20 @@ namespace Orthanc
     request.DataSetType = DIMSE_DATASET_PRESENT;
     strncpy(request.MoveDestination, targetAet.c_str(), DIC_AE_LEN);
 
+    {
+      OFString str;
+      CLOG(TRACE, DICOM) << "Sending Move Request:" << std::endl
+                         << DIMSE_dumpMessage(str, request, DIMSE_OUTGOING, NULL, presID);
+    }
+    
     T_DIMSE_C_MoveRSP response;
     DcmDataset* statusDetail = NULL;
     DcmDataset* responseIdentifiers = NULL;
     OFCondition cond = DIMSE_moveUser(
-      &association_->GetDcmtkAssociation(), presID, &request, dataset, NULL, NULL,
+      &association_->GetDcmtkAssociation(), presID, &request, dataset, /*moveCallback*/ NULL, NULL,
       /*opt_blockMode*/ (parameters_.HasTimeout() ? DIMSE_NONBLOCKING : DIMSE_BLOCKING),
       /*opt_dimse_timeout*/ parameters_.GetTimeout(),
-      &association_->GetDcmtkNetwork(), NULL, NULL,
+      &association_->GetDcmtkNetwork(), /*subOpCallback*/ NULL, NULL,
       &response, &statusDetail, &responseIdentifiers);
 
     if (statusDetail)
@@ -388,6 +408,11 @@ namespace Orthanc
 
     DicomAssociation::CheckCondition(cond, parameters_, "C-MOVE");
 
+    {
+      OFString str;
+      CLOG(TRACE, DICOM) << "Received Final Move Response:" << std::endl
+                         << DIMSE_dumpMessage(str, response, DIMSE_INCOMING);
+    }
     
     /**
      * New in Orthanc 1.6.0: Deal with failures during C-MOVE.
@@ -472,33 +497,28 @@ namespace Orthanc
     }
     
     DcmDataset* dataset = query->GetDcmtkObject().getDataset();
+    assert(dataset != NULL);
 
-    const char* clevel = NULL;
+    const char* clevel = ResourceTypeToDicomQueryRetrieveLevel(level);
     const char* sopClass = NULL;
 
+    DU_putStringDOElement(dataset, DCM_QueryRetrieveLevel, clevel);
+
     switch (level)
     {
       case ResourceType_Patient:
-        clevel = "PATIENT";
-        DU_putStringDOElement(dataset, DCM_QueryRetrieveLevel, "PATIENT");
         sopClass = UID_FINDPatientRootQueryRetrieveInformationModel;
         break;
 
       case ResourceType_Study:
-        clevel = "STUDY";
-        DU_putStringDOElement(dataset, DCM_QueryRetrieveLevel, "STUDY");
         sopClass = UID_FINDStudyRootQueryRetrieveInformationModel;
         break;
 
       case ResourceType_Series:
-        clevel = "SERIES";
-        DU_putStringDOElement(dataset, DCM_QueryRetrieveLevel, "SERIES");
         sopClass = UID_FINDStudyRootQueryRetrieveInformationModel;
         break;
 
       case ResourceType_Instance:
-        clevel = "IMAGE";
-        DU_putStringDOElement(dataset, DCM_QueryRetrieveLevel, "IMAGE");
         sopClass = UID_FINDStudyRootQueryRetrieveInformationModel;
         break;
 
