@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -162,7 +162,7 @@ namespace Orthanc
       assert(response != NULL);
       assert(requestIdentifiers != NULL);
       
-      bzero(response, sizeof(T_DIMSE_C_MoveRSP));
+      memset(response, 0, sizeof(T_DIMSE_C_MoveRSP));
       *statusDetail = NULL;
       *responseIdentifiers = NULL;   
 
