@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -68,7 +68,7 @@ namespace Orthanc
     class MetricsTimer;
 
     IStorageArea&     area_;
-    StorageCache&     cache_;
+    StorageCache*     cache_;
     MetricsRegistry*  metrics_;
 
 #if ORTHANC_ENABLE_CIVETWEB == 1 || ORTHANC_ENABLE_MONGOOSE == 1
@@ -78,8 +78,13 @@ namespace Orthanc
 #endif
 
   public:
-    explicit StorageAccessor(IStorageArea& area,
-                             StorageCache& cache);
+    explicit StorageAccessor(IStorageArea& area);
+
+    StorageAccessor(IStorageArea& area,
+                    StorageCache& cache);
+
+    StorageAccessor(IStorageArea& area,
+                    MetricsRegistry& metrics);
 
     StorageAccessor(IStorageArea& area,
                     StorageCache& cache,
