@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -182,6 +182,32 @@ namespace Orthanc
 #endif
 
 
+#if ORTHANC_ENABLE_CURL == 1
+  int LuaContext::SetHttpTimeout(lua_State *state)
+  {
+    LuaContext& that = GetLuaContext(state);
+
+    // Check the types of the arguments
+    int nArgs = lua_gettop(state);
+    if (nArgs != 1 ||
+        !lua_isnumber(state, 1))    // Timeout
+    {
+      LOG(ERROR) << "Lua: Bad parameters to SetHttpTimeout()";
+    }
+    else
+    {
+      // Configure the HTTP client
+      // Convert to "int" if truncation does not loose precision
+      long timeout = static_cast<long>(lua_tonumber(state, 1));
+
+      that.httpClient_.SetTimeout(timeout);
+    }
+
+    return 0;
+  }
+#endif
+
+
 #if ORTHANC_ENABLE_CURL == 1
   bool LuaContext::AnswerHttpQuery(lua_State* state)
   {
@@ -370,10 +396,6 @@ namespace Orthanc
       const std::string s = value.asString();
       lua_pushlstring(lua_, s.c_str(), s.size());
     }
-    else if (value.isDouble())
-    {
-      lua_pushnumber(lua_, value.asDouble());
-    }
     else if (value.isInt())
     {
       lua_pushinteger(lua_, value.asInt());
@@ -382,6 +404,10 @@ namespace Orthanc
     {
       lua_pushinteger(lua_, value.asUInt());
     }
+    else if (value.isDouble())
+    {
+      lua_pushnumber(lua_, value.asDouble());
+    }
     else if (value.isBool())
     {
       lua_pushboolean(lua_, value.asBool());
@@ -566,6 +592,7 @@ namespace Orthanc
     lua_register(lua_, "HttpPut", CallHttpPut);
     lua_register(lua_, "HttpDelete", CallHttpDelete);
     lua_register(lua_, "SetHttpCredentials", SetHttpCredentials);
+    lua_register(lua_, "SetHttpTimeout", SetHttpTimeout);
 #endif
 
     SetGlobalVariable("_LuaContext", this);
@@ -620,15 +647,6 @@ namespace Orthanc
   }
 
 
-#if ORTHANC_ENABLE_CURL == 1
-  void LuaContext::SetHttpCredentials(const char* username,
-                                      const char* password)
-  {
-    httpClient_.SetCredentials(username, password);
-  }
-#endif
-
-
   void LuaContext::Execute(Json::Value& output,
                            const std::string& command)
   {
