@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,6 +34,8 @@
 
 #include <OrthancServerResources.h>
 
+static const char* ON_HEART_BEAT = "OnHeartBeat";
+
 
 namespace Orthanc
 {
@@ -759,7 +761,8 @@ namespace Orthanc
 
   LuaScripting::LuaScripting(ServerContext& context) : 
     context_(context),
-    state_(State_Setup)
+    state_(State_Setup),
+    heartBeatPeriod_(0)
   {
     lua_.SetGlobalVariable("_ServerContext", &context);
     lua_.RegisterFunction("RestApiGet", RestApiGet);
@@ -782,6 +785,41 @@ namespace Orthanc
     }
   }
 
+  void LuaScripting::HeartBeatThread(LuaScripting* that)
+  {
+    static const unsigned int GRANULARITY = 100;  // In milliseconds
+    
+    const boost::posix_time::time_duration PERIODICITY =
+      boost::posix_time::seconds(that->heartBeatPeriod_);
+    
+    boost::posix_time::ptime next =
+      boost::posix_time::microsec_clock::universal_time() + PERIODICITY;
+    
+    bool shouldStop = false;
+
+    while (!shouldStop)
+    {
+      boost::this_thread::sleep(boost::posix_time::milliseconds(GRANULARITY));
+
+      if (boost::posix_time::microsec_clock::universal_time() >= next)
+      {
+        LuaScripting::Lock lock(*that);
+
+        if (lock.GetLua().IsExistingFunction(ON_HEART_BEAT))
+        {
+          LuaFunctionCall call(lock.GetLua(), ON_HEART_BEAT);
+          call.Execute();
+        }
+
+        next = boost::posix_time::microsec_clock::universal_time() + PERIODICITY;
+      }
+
+      {
+        boost::recursive_mutex::scoped_lock lock(that->mutex_);
+        shouldStop = (that->state_ == State_Done);
+      }
+    }
+  }
 
   void LuaScripting::EventThread(LuaScripting* that)
   {
@@ -829,6 +867,14 @@ namespace Orthanc
     {
       LOG(INFO) << "Starting the Lua engine";
       eventThread_ = boost::thread(EventThread, this);
+      
+      LuaScripting::Lock luaLock(*this);
+
+      if (heartBeatPeriod_ > 0 && luaLock.GetLua().IsExistingFunction(ON_HEART_BEAT))
+      {
+        LOG(INFO) << "Starting the Lua HeartBeat thread with a period of " << heartBeatPeriod_ << " seconds";
+        heartBeatThread_ = boost::thread(HeartBeatThread, this);
+      }
       state_ = State_Running;
     }
   }
@@ -853,6 +899,10 @@ namespace Orthanc
     {
       LOG(INFO) << "Stopping the Lua engine";
       eventThread_.join();
+      if (heartBeatThread_.joinable())
+      {
+        heartBeatThread_.join();
+      }
       LOG(INFO) << "The Lua engine has stopped";
     }
   }
@@ -989,6 +1039,7 @@ namespace Orthanc
 
     std::list<std::string> luaScripts;
     configLock.GetConfiguration().GetListOfStringsParameter(luaScripts, "LuaScripts");
+    heartBeatPeriod_ = configLock.GetConfiguration().GetIntegerParameter("LuaHeartBeatPeriod", 0);
 
     LuaScripting::Lock lock(*this);
 
