@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -75,10 +75,14 @@ namespace Orthanc
     LuaJobManager            jobManager_;
     State                    state_;
     boost::thread            eventThread_;
+    boost::thread            heartBeatThread_;
+    unsigned int             heartBeatPeriod_;
     SharedMessageQueue       pendingEvents_;
 
     static void EventThread(LuaScripting* that);
 
+    static void HeartBeatThread(LuaScripting* that);
+
     void LoadGlobalConfiguration();
 
   public:
