@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,6 +35,7 @@
 #include "../../OrthancFramework/Sources/FileStorage/StorageCache.h"
 #include "../../OrthancFramework/Sources/MultiThreading/Semaphore.h"
 
+#include <boost/date_time/posix_time/posix_time.hpp>
 
 namespace Orthanc
 {
@@ -186,6 +187,11 @@ namespace Orthanc
     static void SaveJobsThread(ServerContext* that,
                                unsigned int sleepDelay);
 
+#if HAVE_MALLOC_TRIM == 1
+    static void MemoryTrimmingThread(ServerContext* that,
+                                     unsigned int intervalInSeconds);
+#endif
+
     void SaveJobsEngine();
 
     virtual void SignalJobSubmitted(const std::string& jobId) ORTHANC_OVERRIDE;
@@ -229,6 +235,7 @@ namespace Orthanc
     SharedMessageQueue  pendingChanges_;
     boost::thread  changeThread_;
     boost::thread  saveJobsThread_;
+    boost::thread  memoryTrimmingThread_;
         
     std::unique_ptr<SharedArchive>  queryRetrieveArchive_;
     std::string defaultLocalAet_;
@@ -241,6 +248,7 @@ namespace Orthanc
     std::unique_ptr<MetricsRegistry>  metricsRegistry_;
     bool isHttpServerSecure_;
     bool isExecuteLuaEnabled_;
+    bool isRestApiWriteToFileSystemEnabled_;
     bool overwriteInstances_;
 
     std::unique_ptr<StorageCommitmentReports>  storageCommitmentReports_;
@@ -261,13 +269,8 @@ namespace Orthanc
 
     StoreResult StoreAfterTranscoding(std::string& resultPublicId,
                                       DicomInstanceToStore& dicom,
-                                      StoreInstanceMode mode);
-
-    void ApplyInternal(ILookupVisitor& visitor,
-                       const DatabaseLookup& lookup,
-                       ResourceType queryLevel,
-                       size_t since,
-                       size_t limit);
+                                      StoreInstanceMode mode,
+                                      bool isReconstruct);
 
     void PublishDicomCacheMetrics();
 
@@ -281,6 +284,8 @@ namespace Orthanc
     DicomModification logsDeidentifierRules_;
     bool              deidentifyLogs_;
 
+    boost::posix_time::ptime serverStartTimeUtc_;
+
   public:
     class DicomCacheLocker : public boost::noncopyable
     {
@@ -341,6 +346,11 @@ namespace Orthanc
                       DicomInstanceToStore& dicom,
                       StoreInstanceMode mode);
 
+    StoreResult TranscodeAndStore(std::string& resultPublicId,
+                                  DicomInstanceToStore* dicom,
+                                  StoreInstanceMode mode,
+                                  bool isReconstruct = false);
+
     void AnswerAttachment(RestApiOutput& output,
                           const std::string& resourceId,
                           FileContentType content);
@@ -370,7 +380,8 @@ namespace Orthanc
                         int64_t& revision,
                         const std::string& instancePublicId,
                         FileContentType content,
-                        bool uncompressIfNeeded);
+                        bool uncompressIfNeeded,
+                        bool skipCache = false);
 
     void SetStoreMD5ForAttachments(bool storeMD5);
 
@@ -477,6 +488,16 @@ namespace Orthanc
       return isExecuteLuaEnabled_;
     }
 
+    void SetRestApiWriteToFileSystemEnabled(bool enabled)
+    {
+      isRestApiWriteToFileSystemEnabled_ = enabled;
+    }
+
+    bool IsRestApiWriteToFileSystemEnabled() const
+    {
+      return isRestApiWriteToFileSystemEnabled_;
+    }
+
     void SetOverwriteInstances(bool overwrite)
     {
       overwriteInstances_ = overwrite;
@@ -539,5 +560,39 @@ namespace Orthanc
     bool IsUnknownSopClassAccepted();
 
     void SetUnknownSopClassAccepted(bool accepted);
+
+    bool ExpandResource(Json::Value& target,
+                        const std::string& publicId,
+                        ResourceType level,
+                        DicomToJsonFormat format,
+                        const std::set<DicomTag>& requestedTags,
+                        bool allowStorageAccess);
+
+    bool ExpandResource(Json::Value& target,
+                        const std::string& publicId,
+                        const DicomMap& mainDicomTags,    // optional: the main dicom tags for the resource (if already available)
+                        const std::string& instanceId,    // optional: the id of an instance for the resource
+                        const Json::Value* dicomAsJson,   // optional: the dicom-as-json for the resource
+                        ResourceType level,
+                        DicomToJsonFormat format,
+                        const std::set<DicomTag>& requestedTags,
+                        bool allowStorageAccess);
+
+    bool ExpandResource(ExpandedResource& target,
+                        const std::string& publicId,
+                        const DicomMap& mainDicomTags,    // optional: the main dicom tags for the resource (if already available)
+                        const std::string& instanceId,    // optional: the id of an instance for the resource
+                        const Json::Value* dicomAsJson,   // optional: the dicom-as-json for the resource
+                        ResourceType level,
+                        const std::set<DicomTag>& requestedTags,
+                        ExpandResourceFlags expandFlags,
+                        bool allowStorageAccess);
+
+    FindStorageAccessMode GetFindStorageAccessMode() const
+    {
+      return findStorageAccessMode_;
+    }
+
+    int64_t GetServerUpTime() const;
   };
 }
