@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -51,7 +51,7 @@ namespace Orthanc
   public:
     explicit DicomMoveScuJob(ServerContext& context) :
       context_(context),
-      query_(true /* this is for worklists */),
+      query_(false  /* this is not for worklists */),
       queryFormat_(DicomToJsonFormat_Short)
     {
     }
@@ -61,6 +61,8 @@ namespace Orthanc
 
     void AddFindAnswer(const DicomMap& answer);
     
+    void AddQuery(const DicomMap& query);
+
     void AddFindAnswer(QueryRetrieveHandler& query,
                        size_t i);
 
