@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -112,7 +112,8 @@ TEST(Versions, ZlibStatic)
 
 TEST(Versions, BoostStatic)
 {
-  ASSERT_STREQ("1_69", BOOST_LIB_VERSION);
+  ASSERT_TRUE(std::string(BOOST_LIB_VERSION) == "1_82" ||
+              std::string(BOOST_LIB_VERSION) == "1_69" /* if USE_LEGACY_BOOST */);
 }
 
 TEST(Versions, CurlStatic)
@@ -166,11 +167,11 @@ TEST(Version, LibIconvStatic)
 #if ORTHANC_ENABLE_SSL == 1
 TEST(Version, OpenSslStatic)
 {
-  // openssl-3.0.1
-  ASSERT_EQ(3 * 0x10000000L +
-            0 * 0x00100000L +
-            1 * 0x00000010L +
-            0 * 0x0000000fL, OPENSSL_VERSION_NUMBER);
+  // openssl-3.1.0
+  // https://www.openssl.org/docs/man3.0/man3/OPENSSL_VERSION_NUMBER.html
+  ASSERT_EQ(3 /* major */ * 0x10000000L +
+            1 /* minor */ * 0x00100000L +
+            0 /* patch */ * 0x00000010L, OPENSSL_VERSION_NUMBER);
 }
 #endif
 
